from math import *
from matplotlib.patches import Polygon
import matplotlib.pyplot as plt

# on decoupe les triangles issus des triangulations d'un polygone régulier

# on utilise un "dictionnaire" : par exemple pav5 et pav6 ci-dessous. Ce dictionnaire doit expliquer les découpages de chaque type de triangle utilisé ("1", "2", etc...)
# n correspond au découpage d'un n-gone régulier
# init pour la configuration initiale : triangle typé dont les sommets sont aux places i,j,k sur le polygone
# les coefficients (barycentriques) : -1 si non utilisé, i pour coeff ui
# les nouveaux sommets sont ajoutés à la liste des trois premiers (indices 3,4, ...)

pav5 = {"n":5,  # parce qu'issu d'un pentagone régulier
        "init": [(2,3,0),"2"], # choix d'un triangle initial : dans le pengone, sommets numéros 2,3 et 0 (ou 5)
        "1":[[(0,1,-1)],[[3,2,0,"2"],[1,2,3,"1"]]], # codes coef pour nouveaux sommets (ici un seul) puis triangles typés
        "2":[[(1,-1,0),(-1,0,1)],[[3,0,1,"2"],[4,3,1,"2"],[2,3,4,"1"]]] # là deux nouveaux sommets : s4 et s5
        }        

pav6 = {"n":6,
        "init": [(0,2,4),"3"],
        "1":[[(2,0,-1)],[[2,0,3,"1"],[1,3,2,"2"]]],
        "2":[[(1,1,-1),(2,-1,0)],[[0,3,4,"1"],[1,2,3,"3"],[2,4,3,"2"]]],
        "3": [[(0,2,-1),(-1,0,2),(2,-1,0)],[[3,0,5,"2"],[4,1,3,"2"],[5,2,4,"2"],[3,4,5,"3"]]]
        }

pav5a = {"n":5,
        "init": [(2,3,0),"2"],
        "1":[[(1,0,-1)],[[2,3,1,"2"],[2,0,3,"1"]]], # codes coef pour nouveau sommet puis triangles typés
        "2":[[(1,-1,0),(-1,0,1)],[[3,0,1,"2"],[4,3,1,"2"],[2,3,4,"1"]]]
        } 

palette = {"1":"white","2":"blue","3":"yellow"}  # les couleurs utilisées...


def u(i,theta): # polynômes de Tchebytchev de seconde espece (ou 0 pour i=-1)
    if i == -1:
        return 0
    else:
        try:
            return(sin((i+1)*theta)/sin(theta))
        except TypeError:
            return(sum([u(j,theta) for j in i]))

def bar(ls,lc): # ls liste de 3 sommets, lc liste de triplets de coefficients
    x1,y1 = ls[0]
    x2,y2 = ls[1]
    x3,y3 = ls[2]
    ns = [] # liste des nouveaux sommets 
    for triplet in lc:
        a,b,c = triplet 
        x = (a*x1+b*x2+c*x3)/(a+b+c)
        y = (a*y1+b*y2+c*y3)/(a+b+c)
        ns.append((x,y))
    return(ns)

def successeurs(triangle,dico):
    n = dico["n"]
    alpha = pi/n
    t = triangle[3] # type
    sommets = triangle[:3]
    infos = dico[t]
    infosSommets = infos[0] # liste des triplets de coefficients
    listeCoeff = []
    for info in infosSommets:
        listeCoeff.append(tuple([u(i, alpha) for i in info ]))
    sommets += bar(triangle,listeCoeff)
    infosTriangles = infos[1]
    Nouveaux = [] # nouveaux triangles typés
    for toto in infosTriangles:
        t = toto[3]
        i1, i2, i3 = toto[:3]
        Nouveaux.append([sommets[i1],sommets[i2],sommets[i3],t])
    return(Nouveaux)  


def diagonales(n):
    return([u(i,pi/n) for i in range(floor((n-1)/2))])

def decoupage(dico,k,debut=None): # dico : type de pavage, k : nb d'itérations ; argument optionel : pour un debut différent de init
    n = dico["n"]
    alpha = pi/n
    s,t = dico["init"]
    if debut is None:
        listeInitiale = [ (cos(2*alpha*i+pi/2),sin(2*alpha*i+pi/2)) for i in s ] + [t] # triangle initial
    else:
        listeInitiale = debut
    if type(listeInitiale[0])==tuple: # un seul triangle dans la liste
        listeInitiale = successeurs(listeInitiale,dico)[:]
        if k==1:
            return(listeInitiale)
        else:
            k=k-1
    for etape in range(k):
        listeFinale = []
        for triangle in listeInitiale:
            listeFinale += successeurs(triangle,dico)
        listeInitiale = listeFinale[:]
    return(listeFinale)
     
def dessin(dico,nb_etapes): # version Tikz
    """ produit le code Tikz"""
    palette = {"1":"white","2":"monrouge","3":"yellow"}
    liste = decoupage(dico,nb_etapes)
    chaine = f'\\begin{{scope}}[scale=1]\n'
    for triangle in liste:
        couleur = palette[triangle[3]]
        s1,s2,s3 = triangle[:3]
        chaine += f'\\draw [fill={couleur}] {s1} -- {s2} -- {s3} -- cycle;\n'
    chaine += '\\end{scope}\n'
    return(chaine)  


def dessinmpost(liste,numfig=1): # version MetaPost
    """ produit le code MetaPost"""
    chaine = 'verbatimtex;\n'
    chaine += '%&pdflatex;\n'
    chaine += '\pdfoutput=0\n'
    chaine += 'etex;\n'
    chaine += 'prologues:=3;\n'
    chaine += '% ici on est en MPS (et couleurs CMYK)\n'
    chaine += 'defaultcolormodel:=7;\n'
    chaine += 'outputtemplate:="%j_%2c_CMJN.mps";\n'
    chaine += 'u:=10cm;\n'
    chaine += 'cmykcolor col,blue,yellow,white;\n'
    chaine += 'col:=(0.66,0.39,0,0.2);\n'
    chaine += 'blue:=(1,1,0,0);\n'
    chaine += 'yellow:=(0,0,1,0);\n'
    chaine += 'white:=(0,0,0,0);\n'
    chaine += f'beginfig({numfig});\n'
    for triangle in liste:
        couleur = palette[triangle[3]]
        s1,s2,s3 = triangle[:3]
        chaine += f'fill u*{s1}--u*{s2}--u*{s3}--cycle withcmykcolor({couleur});\n'
    chaine += 'endfig;\n'
    chaine += '\n'
    chaine += '% ici on passe en SVG (et couleurs RGB)\n'
    chaine += 'currentpicture:=nullpicture;\n'
    chaine += 'defaultcolormodel:=5;\n'
    chaine += 'outputformat:="svg";\n'
    chaine += 'outputtemplate:="%j_%2c_RVB.svg";\n'
    chaine += 'color col,blue,yellow,white;\n'
    chaine += 'col:=(1/255)*(69,124,204);\n'
    chaine += 'blue:=(0,0,1);\n'
    chaine += 'yellow:=(1,1,0);\n'
    chaine += 'white:=(1,1,1);\n'
    chaine += f'beginfig({numfig});\n'
    for triangle in liste:
        couleur = palette[triangle[3]]
        s1,s2,s3 = triangle[:3]
        chaine += f'fill u*{s1}--u*{s2}--u*{s3}--cycle withcolor({couleur});\n'
    chaine += 'endfig;\n'
    chaine += '\n'
    chaine += 'end;\n'
    return(chaine)  


def dessine(dico,iterations): # version matplotlib
    liste = decoupage(dico,iterations)
    fig,ax = plt.subplots(1,1)
    ax.set_ylim([-1,1])
    ax.set_xlim([-1,1])
    ax.set_xticks([])
    ax.set_yticks([])
    for sp in ax.spines.values():
        sp.set_visible(False)
    plt.plot([-1,1,1,-1],[-1,-1,1,1],"white")
    ax.axis('equal')
    for triangle in liste:
        couleur = palette[triangle[3]]
        polygon = Polygon(triangle[:3],closed=True,facecolor=couleur,edgecolor='black',alpha=0.5)
        ax.add_patch(polygon)
    plt.show()


dessine(pav6,6)
